@@ -12,7 +12,7 @@
  * 
  */
 
-
+#include "global/signal_handler.h"
 
 #include "include/types.h"
 #include "common/entity_name.h"
@@ -20,6 +20,7 @@
 #include "common/signal.h"
 #include "common/ceph_argparse.h"
 
+
 #include "msg/Messenger.h"
 #include "mon/MonClient.h"
 
@@ -454,6 +455,10 @@ int MDS::init(int wanted_state)
   monc->wait_auth_rotating(30.0);
 
   mds_lock.Lock();
+  if (want_state == CEPH_MDS_STATE_DNE) {
+    mds_lock.Unlock();
+    return 0;
+  }
 
   timer.init();
 
@@ -578,8 +583,6 @@ void MDS::tick()
 void MDS::beacon_start()
 {
   beacon_send();         // send first beacon
-  
-  //reset_beacon_killer(); // schedule killer
 }
   
 
@@ -1511,15 +1514,19 @@ void MDS::stopping_done()
   request_state(MDSMap::STATE_STOPPED);
 }
 
-  
+void MDS::handle_signal(int signum)
+{
+  assert(signum == SIGINT || signum == SIGTERM);
+  derr << "*** got signal " << sys_siglist[signum] << " ***" << dendl;
+  mds_lock.Lock();
+  suicide();
+  mds_lock.Unlock();
+}
 
 void MDS::suicide()
 {
   assert(mds_lock.is_locked());
-  if (want_state == MDSMap::STATE_STOPPED)
-    state = want_state;
-  else
-    state = CEPH_MDS_STATE_DNE; // whatever.
+  want_state = CEPH_MDS_STATE_DNE; // whatever.
 
   dout(1) << "suicide.  wanted " << ceph_mds_state_name(want_state)
 	  << ", now " << ceph_mds_state_name(state) << dendl;
@@ -1534,7 +1541,7 @@ void MDS::suicide()
     tick_event = 0;
   }
   timer.cancel_all_events();
-  //timer.join();  // this will deadlock from beacon_kill -> suicide
+  //timer.join();
   
   // shut down cache
   mdcache->shutdown();
@@ -1586,8 +1593,15 @@ void MDS::respawn()
 
 bool MDS::ms_dispatch(Message *m)
 {
+  bool ret;
   mds_lock.Lock();
-  bool ret = _dispatch(m);
+  if (want_state == CEPH_MDS_STATE_DNE) {
+    dout(10) << " stopping, discarding " << *m << dendl;
+    m->put();
+    ret = true;
+  } else {
+    ret = _dispatch(m);
+  }
   mds_lock.Unlock();
   return ret;
 }
@@ -1950,6 +1964,8 @@ void MDS::ms_handle_connect(Connection *con)
 {
   Mutex::Locker l(mds_lock);
   dout(0) << "ms_handle_connect on " << con->get_peer_addr() << dendl;
+  if (want_state == CEPH_MDS_STATE_DNE)
+    return;
   objecter->ms_handle_connect(con);
 }
 
@@ -1957,6 +1973,9 @@ bool MDS::ms_handle_reset(Connection *con)
 {
   Mutex::Locker l(mds_lock);
   dout(0) << "ms_handle_reset on " << con->get_peer_addr() << dendl;
+  if (want_state == CEPH_MDS_STATE_DNE)
+    return false;
+
   if (con->get_peer_type() == CEPH_ENTITY_TYPE_OSD) {
     objecter->ms_handle_reset(con);
   } else if (con->get_peer_type() == CEPH_ENTITY_TYPE_CLIENT) {
@@ -1979,6 +1998,8 @@ void MDS::ms_handle_remote_reset(Connection *con)
 {
   Mutex::Locker l(mds_lock);
   dout(0) << "ms_handle_remote_reset on " << con->get_peer_addr() << dendl;
+  if (want_state == CEPH_MDS_STATE_DNE)
+    return;
   objecter->ms_handle_remote_reset(con);
 }
 
@@ -1987,6 +2008,8 @@ bool MDS::ms_verify_authorizer(Connection *con, int peer_type,
 			       bool& is_valid)
 {
   Mutex::Locker l(mds_lock);
+  if (want_state == CEPH_MDS_STATE_DNE)
+    return false;
 
   AuthAuthorizeHandler *authorize_handler =
     authorize_handler_registry->get_handler(protocol);
