@@ -562,6 +562,13 @@ protected:
 
   epoch_t last_peering_reset;
 
+
+  /* heartbeat peers */
+public:
+  Mutex heartbeat_peer_lock;
+  set<int> heartbeat_peers;
+
+protected:
   /**
    * BackfillInterval
    *
@@ -667,7 +674,7 @@ public:
     return false;
   }
   
-  bool is_all_uptodate() const;
+  bool needs_recovery() const;
 
   void generate_past_intervals();
   void trim_past_intervals();
@@ -741,6 +748,8 @@ public:
 
   void purge_strays();
 
+  void update_heartbeat_peers();
+
   Context *finish_sync_event;
 
   void finish_recovery(ObjectStore::Transaction& t, list<Context*>& tfin);
@@ -822,6 +831,11 @@ public:
       rctx = 0;
     }
 
+    struct QueryState : boost::statechart::event< QueryState > {
+      Formatter *f;
+      QueryState(Formatter *f) : f(f) {}
+    };
+
     struct MInfoRec : boost::statechart::event< MInfoRec > {
       int from;
       pg_info_t &info;
@@ -974,10 +988,12 @@ public:
       void exit();
 
       typedef boost::mpl::list <
+	boost::statechart::custom_reaction< QueryState >,
 	boost::statechart::custom_reaction< AdvMap >,
 	boost::statechart::custom_reaction< ActMap >,
 	boost::statechart::transition< boost::statechart::event_base, Crashed >
 	> reactions;
+      boost::statechart::result react(const QueryState& q);
       boost::statechart::result react(const AdvMap&);
       boost::statechart::result react(const ActMap&);
     };
@@ -989,9 +1005,11 @@ public:
       void exit();
 
       typedef boost::mpl::list <
+	boost::statechart::custom_reaction< QueryState >,
 	boost::statechart::custom_reaction< AdvMap >,
 	boost::statechart::transition< boost::statechart::event_base, Crashed >
 	> reactions;
+      boost::statechart::result react(const QueryState& q);
       boost::statechart::result react(const AdvMap&);
     };
 
@@ -1043,12 +1061,14 @@ public:
     struct WaitActingChange : boost::statechart::state< WaitActingChange, Primary>,
 			      NamedState {
       typedef boost::mpl::list <
+	boost::statechart::custom_reaction< QueryState >,
 	boost::statechart::custom_reaction< AdvMap >,
 	boost::statechart::custom_reaction< MLogRec >,
 	boost::statechart::custom_reaction< MInfoRec >,
 	boost::statechart::custom_reaction< MNotifyRec >
 	> reactions;
       WaitActingChange(my_context ctx);
+      boost::statechart::result react(const QueryState& q);
       boost::statechart::result react(const AdvMap&);
       boost::statechart::result react(const MLogRec&);
       boost::statechart::result react(const MInfoRec&);
@@ -1072,9 +1092,11 @@ public:
       void exit();
 
       typedef boost::mpl::list <
+	boost::statechart::custom_reaction< QueryState >,
 	boost::statechart::transition< Activate, Active >,
 	boost::statechart::custom_reaction< AdvMap >
 	> reactions;
+      boost::statechart::result react(const QueryState& q);
       boost::statechart::result react(const AdvMap &advmap);
     };
 
@@ -1083,6 +1105,7 @@ public:
       void exit();
 
       typedef boost::mpl::list <
+	boost::statechart::custom_reaction< QueryState >,
 	boost::statechart::custom_reaction< ActMap >,
 	boost::statechart::custom_reaction< AdvMap >,
 	boost::statechart::custom_reaction< MInfoRec >,
@@ -1090,6 +1113,7 @@ public:
 	boost::statechart::custom_reaction< MLogRec >,
 	boost::statechart::custom_reaction< RecoveryComplete >
 	> reactions;
+      boost::statechart::result react(const QueryState& q);
       boost::statechart::result react(const ActMap&);
       boost::statechart::result react(const AdvMap&);
       boost::statechart::result react(const MInfoRec& infoevt);
@@ -1103,11 +1127,13 @@ public:
       void exit();
 
       typedef boost::mpl::list <
+	boost::statechart::custom_reaction< QueryState >,
 	boost::statechart::custom_reaction< ActMap >,
 	boost::statechart::custom_reaction< MQuery >,
 	boost::statechart::custom_reaction< MInfoRec >,
 	boost::statechart::custom_reaction< MLogRec >
 	> reactions;
+      boost::statechart::result react(const QueryState& q);
       boost::statechart::result react(const MInfoRec& infoevt);
       boost::statechart::result react(const MLogRec& logevt);
       boost::statechart::result react(const ActMap&);
@@ -1143,9 +1169,11 @@ public:
       void get_infos();
 
       typedef boost::mpl::list <
+	boost::statechart::custom_reaction< QueryState >,
 	boost::statechart::transition< GotInfo, GetLog >,
 	boost::statechart::custom_reaction< MNotifyRec >
 	> reactions;
+      boost::statechart::result react(const QueryState& q);
       boost::statechart::result react(const MNotifyRec& infoevt);
     };
 
@@ -1163,9 +1191,11 @@ public:
       void exit();
 
       typedef boost::mpl::list <
+	boost::statechart::custom_reaction< QueryState >,
 	boost::statechart::custom_reaction< MLogRec >,
 	boost::statechart::custom_reaction< GotLog >
 	> reactions;
+      boost::statechart::result react(const QueryState& q);
       boost::statechart::result react(const MLogRec& logevt);
       boost::statechart::result react(const GotLog&);
     };
@@ -1179,9 +1209,11 @@ public:
       void exit();
 
       typedef boost::mpl::list <
+	boost::statechart::custom_reaction< QueryState >,
 	boost::statechart::custom_reaction< MLogRec >,
 	boost::statechart::transition< NeedUpThru, WaitUpThru >
 	> reactions;
+      boost::statechart::result react(const QueryState& q);
       boost::statechart::result react(const MLogRec& logevt);
     };
 
@@ -1190,9 +1222,11 @@ public:
       void exit();
 
       typedef boost::mpl::list <
+	boost::statechart::custom_reaction< QueryState >,
 	boost::statechart::custom_reaction< ActMap >,
 	boost::statechart::custom_reaction< MLogRec >
 	> reactions;
+      boost::statechart::result react(const QueryState& q);
       boost::statechart::result react(const ActMap& am);
       boost::statechart::result react(const MLogRec& logrec);
     };
@@ -1222,6 +1256,7 @@ public:
     void handle_recovery_complete(RecoveryCtx *ctx);
     void handle_create(RecoveryCtx *ctx);
     void handle_loaded(RecoveryCtx *ctx);
+    void handle_query_state(Formatter *f);
   } recovery_state;
 
 
@@ -1239,6 +1274,7 @@ public:
     need_up_thru(false),
     need_flush(false),
     last_peering_reset(0),
+    heartbeat_peer_lock("PG::heartbeat_peer_lock"),
     backfill_target(-1),
     pg_stats_lock("PG::pg_stats_lock"),
     pg_stats_valid(false),
@@ -1294,6 +1330,9 @@ public:
 
   bool  is_empty() const { return info.last_update == eversion_t(0,0); }
 
+  void init(int role, vector<int>& up, vector<int>& acting, pg_history_t& history,
+	    ObjectStore::Transaction *t);
+
   // pg on-disk state
   void do_pending_flush();
 
@@ -1386,6 +1425,9 @@ public:
   virtual void do_backfill(OpRequest *op) = 0;
   virtual bool snap_trimmer() = 0;
 
+  virtual int do_command(vector<string>& cmd, ostream& ss,
+			 bufferlist& idata, bufferlist& odata) = 0;
+
   virtual bool same_for_read_since(epoch_t e) = 0;
   virtual bool same_for_modify_since(epoch_t e) = 0;
   virtual bool same_for_rep_modify_since(epoch_t e) = 0;
